######################################################################################################
###################  R code supplementary to Wensink, Ahrenfeldt, Mller  ############################
###################  last edited 23/12/2020 (by Wensink)                  ############################
######################################################################################################

##############################################################
########################  Figure 1  ##########################
##############################################################

# rm(list=ls()) # clears global environment

# The first function models prescription count as a function of age.
# It consists of three parts such that the first derivative is continuous for all x.

# adapt the working directory as appropriate, note forward slashes including at end
setwd("C:/Users/mwensink/OneDrive - Syddansk Universitet/WensinkAhrenfeldtMoeller/")

getMedUse<-function(x){                  # x is going to be age
  ifelse(x<30, out<-0.6,                 # the first part is flat, pill count =0.6 up to age 30
    ifelse(x>55, out<-1.85+(x-55)*0.1,   # the last part is a linear increase by 1 pill per 10 years from age 55
      out<-(x-30)^2*2/1000+0.6           # from ages 30 through 55 there is a quadratic increase
    )
  )
  return(out) # function picks "out" depending on x and then returns out
}

# The second function samples paternal ages for a birth year given mean and standard deviation assuming a Normal distribution.
# In reality, the age distribution is skewed towards higher ages, but this simplifying assumption does not change the story.

sampleFunction<-function(m,s){ 
  out<-rnorm(n=50000, mean=m, sd=s) # a typical birth year is 50,000 or more
  return(out)
}

ages<-seq(0,81,0.01) # 81 is the upper age limit in the data used for Wensink et al. in Scandinavian Journal of Public Health (forthcoming)
pillCount<-NULL # initiates y and leaves the choice of class (vector, matrix, list etc) free (for now, will turn into a vector)
for(i in 1:length(ages)){
  pillCount[i]<-getMedUse(ages[i])
}

# simple base R plot for age pattern (Figure 1a)
plot(ages,pillCount, ylab="Mean number of pre-conception medications", xlab="Paternal age")

# make nicer plot (for publication, Figure 1a)
library(ggplot2) # loads the popular ggplot2 graphics package (run install.packages("ggplot2") if not installed)
ggplot <- function(...) { ggplot2::ggplot(...) + theme_bw() } # sets a simpler theme (can be omitted)

data<-cbind.data.frame(ages,pillCount)

ggplot(data)+geom_point(aes(ages,pillCount,colour='#00BFC4'),size=2)+
  ylab("Mean number of prescriptions") + xlab("Age") + ggtitle("") +
  theme(
  #  panel.grid.major = element_blank(), # if run, this line and the next remove the grid
  #  panel.grid.minor = element_blank(),
    aspect.ratio = 0.7,
    legend.position="n",
    axis.title.y = element_text(vjust=.8, size = rel(1.4)),
    axis.title.x = element_text(vjust=.2, size = rel(1.4)),
    axis.text.y = element_text(hjust = 1,size = rel(1.4)),
    axis.text.x = element_text(hjust = 0.5,size = rel(1.4)))

ggsave(file="Figure 1A.pdf", width = 13, height = 7, scale=0.6) # saves last plot in same directory as the code
ggsave(file="Figure 1A.tiff", width = 13, height = 7, scale=0.6)

# now Figure 1b
means=seq(from=30, to=33, by=0.1)            # the mean trend goes from 30 to 33 for both cases we investigate
sds1=c(seq(from=5, to=7, by=0.1),rep(7,10))  # the sd increases linearly up to year 21, but then remains constant (in our projections)
sds2=seq(from=5,to=8,by=0.1)                 # now the sd trend does continue after year 21
meanMedUse1=0:30                            # these are going to be for the broken sd trend, i.e. sds1
meanMedUse2=0:30                            # these are going to be for the continued sd trend, i.e. sds2

# now put the whole thing in a single dataframe:
dta<-cbind.data.frame(means,sds1)
dta<-cbind(dta,sds2)
dta<-cbind(dta,meanMedUse1)
dta<-cbind(dta,meanMedUse2) 
dta

# now we need to calculate mean prescription count.
# numerical note: a Normal distribution has support on the entire real line.
# However, the probability for each of these Normal distributions of drawing an
# age above 81 or below 0 are negligible. In any event, an age below 0 will get
# pill count 0.6, while an age above 81 will be modelled by the last part
# of function getMedUse (see above), so this may not be altogether unrealistic.

# first for the broken sd trend, i.e. using sds1
for(i in 1:nrow(dta)){
  ages<-sampleFunction(dta$means[i],dta$sds1[i]) # this creates ages with mean and sd as specified
  medUse<-NULL # initiates object
  for(j in 1:length(ages)){
    medUse[j]<-getMedUse(ages[j]) # calculates pill count for each sampled age
  }
  dta$meanMedUse1[i]<-mean(medUse) # reports mean pill count for this combination of mean and sd
}

# now for the continued sd trend, i.e. using sds2
for(i in 1:nrow(dta)){
  ages<-sampleFunction(dta$means[i],dta$sds2[i]) # this creates ages with mean and sd as specified
  medUse<-NULL # initiates object
  for(j in 1:length(ages)){
    medUse[j]<-getMedUse(ages[j]) # calculates pill count for each sampled age
  }
  dta$meanMedUse2[i]<-mean(medUse) # reports mean pill count for this combination of mean and sd
}

# A simple plot in base R:
plot(dta$meanMedUse1, ylab="Mean Number of Pre-conception Medications", xlab="Time",ylim=c(0.62,0.72))
points(dta$meanMedUse2,col="red") # notice that up to year 21, the trends are the same up to sampling variability
abline(v=21) # break point of the trend

# Nicer plot in ggplot2
library(ggplot2)

dta$year<-1:31

ggplot(dta)+geom_point(aes(year,meanMedUse2,colour='F8766D'),size=2)+
  geom_point(aes(year,meanMedUse1,colour='00BFC4'),size=c(rep(-1,21),rep(2,10)))+
  labs(color = "SD trend\n")+
  scale_color_hue(labels = c("variability trend broken", "variability trend continued"))+
  ylab("Mean number of prescriptions") + xlab("Year") + ggtitle("") +
  theme(
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    aspect.ratio = 0.7,
    legend.position=c(0.65,0.15),
    legend.title = element_blank(),
        legend.text = element_text(size = rel(1.25)),
    axis.title.y = element_text(vjust=.8, size = rel(1.4)),
    axis.title.x = element_text(vjust=.2, size = rel(1.4)),
    axis.text.y = element_text(hjust = 1,size = rel(1.4)),
    axis.text.x = element_text(hjust = 0.5,size = rel(1.4)))

ggsave(file="Figure 1B.pdf", width = 13, height = 7, scale=0.6)
ggsave(file="Figure 1B.tiff", width = 13, height = 7, scale=0.6)


##############################################################
########################  Figure 2  ##########################
##############################################################
# rm(list=ls()) # clears working environment
# First read the data, which are freely available at mortality.org (registration required, so we were unable to include this).
# This is the lifetable for Danish females, 1x1:
# "Denmark, Life tables (period 1x1), Females	Last modified: 20 Mar 2020;  Methods Protocol: v6 (2017)"

data<-read.delim("fltper_1x1.txt", header=TRUE, skip=2, sep="") # file should be in same directory as R code / working directory
class(data$Age)                           # character
data$Age<-as.numeric(data$Age)            # warns "NAs introduced by coercion" because "110+" is not recognizable as a number
all(which(is.na(data$Age)==1)/111==1:185) # TRUE, meaning that these are indeed the 110+ ONLY!
data[is.na(data$Age)==1,]$Age<-111        # assume that people who died at 100 or above died at 111.
data<-data[data$Age>64,]                  # truncate at age 65
summary(data$Year)                        # lifetable goes from 1835 to 2019
data<-data[data$Year>1970,]               # truncate at year 1970
data$Age<-data$Age-64.5                   # take age from age 65 onwards, assume that people die halfway the year

dta<-data.frame(cbind(1971:2019,0,0,0,0,0,0)) # dataframe of years, rest filled with zeros
colnames(dta)<-c("year","mean","Exsqrt","sd","propDiedBelow70","propDiedBelow75","propDiedBelow80")
for(i in 1:length(dta$year)){
  dta$mean[i]<-(data[data$Year==dta$year[i],]$dx %*% data[data$Year==dta$year[i],]$Age)/   # life expectancy..
    data[data$Year==dta$year[i],]$lx[1]                                                    # ..conditioned no being alive at age 65
}
dta$mean
# Compare with the official figures from mortality.org, which makes more elegant (but more elaborative) assumptions about when in an interval people die:
data[data$Age==0.5,]$ex
plot(dta$mean, data[data$Age==0.5,]$ex)     # same thing
round(dta$mean,2)==data[data$Age==0.5,]$ex  # true most of the time, negligible differences for the two years not equal

# Now we need the variance, which is E[X^2]-E[X]^2. First we calculate E[X^2], X being age at death 
for(i in 1:length(dta$year)){
  dta$Exsqrt[i]<-(data[data$Year==dta$year[i],]$dx %*% (data[data$Year==dta$year[i],]$Age)^2)/
    data[data$Year==dta$year[i],]$lx[1]
}

dta$sd<-sqrt(dta$Exsqrt-dta$mean^2) # The variance is given by dta$Exsqrt-dta$mean^2, its square root is sd
plot(dta$year,dta$sd)               # simple exploratory plot
plot(dta$year,dta$mean)             # simple exploratory plot
dta$coef<-dta$sd/dta$mean           # express standard deviation proportional to the mean
plot(dta$year,dta$coef)             # simple exploratory plot

for(i in 1:length(dta$year)){
  dta$propDiedBelow70[i]<-data[data$Year==dta$year[i] & data$Age<5,]$dx %*% rep(1,5)/ # this just takes the sum
    data[data$Year==dta$year[i],]$lx[1]                                               # and expresses this as proportion
}
plot(dta$year,dta$propDiedBelow70) # simple plot in base R


for(i in 1:length(dta$year)){
  dta$propDiedBelow75[i]<-data[data$Year==dta$year[i] & data$Age<10,]$dx %*% rep(1,10)/
    data[data$Year==dta$year[i],]$lx[1]
}
plot(dta$year,dta$propDiedBelow75)

for(i in 1:length(dta$year)){
  dta$propDiedBelow80[i]<-data[data$Year==dta$year[i] & data$Age<15,]$dx %*% rep(1,15)/
    data[data$Year==dta$year[i],]$lx[1]
}
plot(dta$year,dta$propDiedBelow80)

for(i in 1:length(dta$year)){
  dta$propDiedBelow90[i]<-data[data$Year==dta$year[i] & data$Age<25,]$dx %*% rep(1,25)/
    data[data$Year==dta$year[i],]$lx[1]
}
plot(dta$year,dta$propDiedBelow90)

dta$propDiedAbove90<-1-dta$propDiedBelow90  # those who didn't die before 90 died at higher ages

#plot all in once with base R
par(mfrow=c(2,2))
plot(dta$year,dta$mean, xlab="year", ylab="a) life expectancy age 65")
plot(dta$year,dta$sd, xlab="year", ylab="b) standard deviation age 65")
plot(dta$year,dta$propDiedBelow75, xlab="year", ylab="c) proportion died before age 75")
plot(dta$year,dta$propDiedAbove90, xlab="year", ylab="d)proportion died after age 90")

# nicer plot in ggplot2
par(mfrow=c(1,1))
dev.off()
library(ggplot2)
ggplot <- function(...) { ggplot2::ggplot(...) + theme_bw() } # sets a simpler theme (can be omitted)


ggplot(dta)+geom_point(aes(year,mean,colour='F8766D'),size=2)+
  geom_point(aes(year,sd/0.6,colour='00BFC4'),size=2)+
  scale_color_hue(labels = c("sd", "mean"))+
  scale_y_continuous(sec.axis = sec_axis(~ . * 0.6, name = "Standard deviation (sd)"))+
  ylab("Remaining life expectancy (mean)") + xlab("Year") + ggtitle("") +
  theme(
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    aspect.ratio = 0.7,
    legend.position=c(0.8,0.45),
    legend.title = element_blank(),
    legend.text = element_text(size = rel(1.25)),
    axis.title.y = element_text(vjust=.8, size = rel(1.4)),
    axis.title.x = element_text(vjust=.2, size = rel(1.4)),
    axis.text.y = element_text(hjust = 1,size = rel(1.4)),
    axis.text.x = element_text(hjust = 0.5,size = rel(1.4)))

ggsave(file="Figure 2A.pdf", width = 13, height = 7, scale=0.6)
ggsave(file="Figure 2A.tiff", width = 13, height = 7, scale=0.6)


ggplot(dta)+geom_point(aes(year,propDiedAbove90,colour='F8766D'),size=2)+
  geom_point(aes(year,propDiedBelow75/0.6,colour='00BFC4'),size=2)+
  scale_color_hue(labels = c("before 75", "after 90"))+
  scale_y_continuous(sec.axis = sec_axis(~ . * 0.6, name = "Proportion died before 75"))+
  ylab("Proportion died after 90") + xlab("Year") + ggtitle("") +
  theme(
    panel.grid.major = element_blank(),
    panel.grid.minor = element_blank(),
    aspect.ratio = 0.7,
    legend.position=c(0.65,0.15),
    legend.title = element_blank(),
    legend.text = element_text(size = rel(1.25)),
    axis.title.y = element_text(vjust=.8, size = rel(1.4)),
    axis.title.x = element_text(vjust=.2, size = rel(1.4)),
    axis.text.y = element_text(hjust = 1,size = rel(1.4)),
    axis.text.x = element_text(hjust = 0.5,size = rel(1.4)))

ggsave(file="Figure 2B.pdf", width = 13, height = 7, scale=0.6)
ggsave(file="Figure 2B.tiff", width = 13, height = 7, scale=0.6)